/*
 * Copyright (C) Michael Fowler 1998 under Gnu Public License
 * http://www.gnu.org/copyleft/gpl.html
 *
 * BrownianApplet.java
 * Drew Dolgert October 1998
 * This applet shows Brownian Motion in two panels.  One is the view of
 * a dust mote in a microscope.  The other is the microscopic explanation
 * for its motion.
 */
 
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class BrownianApplet extends Applet implements ActionListener
{
	Brownian micro;
	Scope sc;
	Button stopButton, resetButton;
	static final String sResource = "BrownianStrings";
	String sStart, sStop, sReset;

	public void init()
	{
		String sParam;
		int nBalls;
		double dMassRatio, dAverageVelocity;
		ResourceBundle rb;

		// The resource bundle contains strings in various languages.
		// See the CollisionStrings*.properties files.
		String sUseRB = this.getParameter("international");
		rb = null;
		if ((sUseRB == null) || !sUseRB.equalsIgnoreCase("no")) {
			String sLang = this.getParameter("language");
			String sCountry = this.getParameter("country");
			try {
				if (sLang == null)
					rb = ResourceBundle.getBundle(sResource, Locale.getDefault());
				else {
					Locale thisloc;
					if (sCountry == null)
						thisloc = new Locale(sLang,"");
					else
						thisloc = new Locale(sLang,sCountry);
					rb = ResourceBundle.getBundle(sResource,thisloc);
				}
			} catch (java.lang.ClassFormatError e) {
				System.out.println("ClassFormatError: "+e.getMessage());
				rb = null;
			} catch (MissingResourceException e) {
				System.out.println("Missing Resource: "+e.getMessage());
				rb = null;
			}
		}
		if (rb!=null) {
			try {
				sStart = rb.getString("start");
				sStop = rb.getString("stop");
				sReset = rb.getString("reset");
			} catch (MissingResourceException e) {
				System.out.println("Missing Resource: "+e.getMessage());
				sStart = "Start";
				sStop = "Stop";
				sReset = "Reset";
			}
		} else {
			sStart = "Start";
			sStop = "Stop";
			sReset = "Reset";
		}
		// Load parameters needed for creating the two main panels.
		sParam = this.getParameter("nballs");
		if (sParam != null) {
			try {
				nBalls = Integer.parseInt(sParam);
			} catch (NumberFormatException e) {
				nBalls = 40;
				System.out.println("nballs should be an integer number of balls in the box.");
			}
		} else nBalls = 40;
		sParam = this.getParameter("massratio");
		if (sParam != null) {
			try {
				dMassRatio = Double.valueOf(sParam).doubleValue();
			} catch (NumberFormatException e) {
				dMassRatio = 20;
				System.out.println("massratio is the floating point ratio of the mass of "+
								"the big blue ball to the red ones.");
			}
		} else dMassRatio = 20;
		sParam = this.getParameter("averagevelocity");
		if (sParam != null) {
			try {
				dAverageVelocity = Double.valueOf(sParam).doubleValue();
			} catch (NumberFormatException e) {
				dAverageVelocity = 5;
				System.out.println("averagevelocity should be a floating point value.");
			}
		} else dAverageVelocity = 5;

		this.setLayout(new BorderLayout());
	    Panel p1 = new Panel(new GridLayout(1,2));
		sc = new Scope();
		micro = new Brownian(nBalls,dAverageVelocity,dMassRatio); // ave Velocity, Mass Ratio

		sParam = this.getParameter("magnification");
		if (sParam != null) {
			try {
				double dMagnification = Double.valueOf(sParam).doubleValue();
				sc.setMagnification(dMagnification);
			} catch (NumberFormatException e) {
				System.out.println("averagevelocity should be a floating point value.");
			}
		}
		sParam = this.getParameter("threadsleep");
		if (sParam != null) {
			try {
				int threadsleep = Integer.parseInt(sParam);
				System.out.println("sleep "+threadsleep);
				micro.setThreadSleep(threadsleep);
			} catch (NumberFormatException e) {
				System.out.println("Threadsleep should be an integer number of milliseconds to sleep.");
			}
		}
		micro.setBackground(new Color(255,248,210));
		micro.setScope(sc);
		p1.add(sc);
		p1.add(micro);
		this.add(p1,"Center");
		Panel p2 = new Panel();
		stopButton = new Button(sStop);
		stopButton.setActionCommand("stop");
		stopButton.addActionListener(this);
		p2.add(stopButton);
		resetButton = new Button(sReset);
		resetButton.setActionCommand("reset");
		resetButton.addActionListener(this);
		p2.add(resetButton);
		this.add(p2,"South");
	}

	public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals("reset")) {
            micro.reset();
            sc.reset();
        } else if (e.getActionCommand().equals("stop")) {
            stopButton.setLabel(sStart);
            stopButton.setActionCommand("start");
			stopButton.getParent().invalidate();
			stopButton.getParent().validate();
            micro.stop();
        } else if (e.getActionCommand().equals("start")) {
            stopButton.setLabel(sStop);
            stopButton.setActionCommand("stop");
			stopButton.getParent().invalidate();
			stopButton.getParent().validate();
            micro.start();
        }

	}

	public String getAppletInfo() {
		return "BrownianApplet v. 1.0.  Written by Drew Dolgert for Dr. Michael Fowler.\n"+
		"Copyright (C) Michael Fowler mf1i@virginia.edu 1998\n"+
		"Under Gnu Public License http://www.gnu.org/copyleft/gpl.html";
	}

	static final String[][] parameterInfo = {
			{"nballs","integer [40]","Number of red balls."},
			{"massratio","double [20.0]","Ratio of the mass of the big blue ball to the red ones."},
			{"averagevelocity","double [5.0]","Average velocity of the red balls in pixels per timestep."},
			{"threadsleep","integer [20]","Number of milliseconds the gas thread should sleep."},
			{"magnification","double [10.0]","The magnification of the microscope view over "+
			        "over the atomic view."}
		};
		
	public String[][] getParameterInfo() {
		return parameterInfo;
	}

	public void start()
	{
		micro.start();
	}

	public void stop()
	{
		micro.stop();
	}
}
		
