/*
 * Copyright (C) Michael Fowler 1998 under Gnu Public License
 * http://www.gnu.org/copyleft/gpl.html
 *
 * Scope.java
 * This is the view of the dust particle through a microscope where
 * you can't see the molecules.  The Brownian object tells it where
 * the particle is.
 */

import java.awt.*;

public class Scope extends Canvas
{
	int pX, pY; // current x and y
	int pNewX, pNewY; // new x and y
	int rh,rl; // width and height of the canvas.
	boolean bStarted; // Whether our "setXY" method has been called.
	boolean bBackGroundPainted; // Whether the background is valid.
	boolean bInside; // false when the particle is off the screen.
    int pMinRadius; // Outside radius of microscope view.
	Dimension thisSize; // Canvas size.
	double dMagnification = 10; // Magnification over atomic view.
	Image offImage, offImageTwo; // TWO offscreen images
	Graphics offscreen, offscreentwo; // and their graphics ports.
    int pBallRadius = 20; // This should be set by the Brownian.class

	public Scope()
	{
		bStarted = false;
		bBackGroundPainted = false;
		bInside = false;
		changeSize();
	}

    public void setMagnification(double m)
    {
        dMagnification = m;
        changeSize();
    }

    public void reset()
    {
        bBackGroundPainted = false;
        bStarted = false;
    }
	
	public void update(Graphics g) { paint(g); }

	public void paint(Graphics g)
	{
		if (!thisSize.equals(getSize())) changeSize();
		if (offscreen==null || offImageTwo==null || thisSize.height<1 ||
				thisSize.width<1) return;

		// Paint the background on the first offscreen image.
        if (!bBackGroundPainted) {
        	g.setClip(0,0,thisSize.width,thisSize.height);
            offscreen.clearRect(0,0,thisSize.width,thisSize.height);
            paintScope(offscreen);
            bBackGroundPainted = true;
        }
		// If we have an x-y coordinate for the particle
		if (bStarted) {
			// If the particle is inside the circle.
			if (bInside) {
				// Draw the particle trail on the first offscreen image.
	    		offscreen.setColor(Color.black);
			    offscreen.drawLine(pX,pY,pNewX, pNewY);
			    // copy that image to the second offscreen image.
	   			offscreentwo.drawImage(offImage,0,0,this);
	   			// Draw the blue ball on the second offscreen image.
	    		offscreentwo.setColor(Color.blue);
	    		offscreentwo.fillOval(pNewX-pBallRadius,pNewY-pBallRadius,2*pBallRadius,2*pBallRadius);
	    		g.drawImage(offImageTwo,0,0,this);
			} else
				g.drawImage(offImage,0,0,this);
    		pX = pNewX;
	    	pY = pNewY;
		}
	}

	// Paint the background to look like the ocular of a microscope.
	// It is a filled oval shell with a pale yellow inside.
	// You can't draw a filled oval shell, so we draw many successively smaller
	// ovals.  We vary the color to make the center of the shell lighter
	// (cheap 3D).
	public void paintScope(Graphics g)
	{
	    int i, rad, lines;
	    double a;
	    
	    if (thisSize.height>thisSize.width) rad = thisSize.width/2;
	    else rad = thisSize.height/2;
	    lines = 25;
	    a = .4/(lines*lines/4);
		// First fill the whole circle with dark green because the successive
	    // ovals don't necessarily overlap perfectly.  I'd rather have little
	    // stray green dots than little stray white dots.
        g.setColor(Color.getHSBColor(.481f,.973f,(float)(0.71-a*(lines/2)*(lines/2))));
	    g.fillOval(thisSize.width/2-rad,thisSize.height/2-rad,2*rad,2*rad);
		// Then draw the oval shell itself.
	    for (i=0; i<lines; i++) {
	        g.setColor(Color.getHSBColor(.481f,.973f,(float)(0.71-a*(i-lines/2)*(i-lines/2))));
	        g.drawOval(thisSize.width/2-rad+i,thisSize.height/2-rad+i,2*(rad-i),2*(rad-i));
	    }
        pMinRadius = rad-lines;
		// Lastly, fill the center with a pale yellow for that eggs and ketchup feel.
	    g.setColor(new Color(255,248,210));
	    g.fillOval(thisSize.width/2-pMinRadius,
	            thisSize.height/2-pMinRadius,2*pMinRadius,2*pMinRadius);
	}

	// Brownian calls this to set the current position of the particle.
	public void setXY(int x, int y)
	{
		double dBallDx;
		
		pNewX = (int) Math.round(x/dMagnification)+thisSize.width/2;
		pNewY = thisSize.height/2-(int) Math.round(y/dMagnification);
		if (thisSize.height>0 && !bStarted) {
			pX = pNewX;
			pY = pNewY;
			bStarted = true;
			repaint();
		}
		int dx = pNewX-thisSize.width/2;
		int dy = pNewY-thisSize.height/2;
		// The fudging below handles when the ball leaves and
		// exits the center circle.  We only draw a line from the
		// last position to this one if both were inside the circle.
		// If the ball re-enters the circle, we just put a dot where
		// the ball is now by making the beginning and end of the line
		// the same point.
		boolean bNewInside;
		dBallDx = Math.sqrt(dx*dx+dy*dy);
        if (dBallDx>pMinRadius) bNewInside = false;
        else bNewInside = true;
        if (bNewInside && !bInside) {
            pX = pNewX; pY = pNewY;
        }
        bInside = bNewInside;
        if (!bBackGroundPainted) repaint();
        else {
			repaint();
        }
	}

	private void changeSize()
	{
		// When the size changes, redraw everything.
		thisSize = this.getSize();
		if (thisSize != null && thisSize.width>0) rl = thisSize.width;
		else rl = 300;
		if (thisSize != null && thisSize.height>0) rh = thisSize.height;
		else rh = 300;
		pBallRadius = (int) Math.round(20/dMagnification);
        bBackGroundPainted = false;
        bStarted = false;
        bInside = false;
	    offImage = createImage(thisSize.width,thisSize.height);
	    if (offImage!=null) {
	    	offscreen=offImage.getGraphics();
	    	offscreen.setClip(0,0,thisSize.width,thisSize.height);
	    }
	    offImageTwo = createImage(thisSize.width,thisSize.height);
	    if (offImageTwo!=null) {
	    	offscreentwo = offImageTwo.getGraphics();
	    	offscreentwo.setClip(0,0,thisSize.width,thisSize.height);
	    }
	    repaint();
	}

	public Dimension getPreferredSize() {
		return new Dimension(rl,rh);
	}

	public Dimension getMinimumSize() {
		return new Dimension(40,40);
	}
}
